import logging
from youtube_dl import YoutubeDL

logger = logging.getLogger(__name__)


class MyLogger(object):
    def debug(self, msg):
        pass

    def warning(self, msg):
        pass

    def error(self, msg):
        print(msg)


def ydl_get_entries(search_term):
    try:
        ydl_opts = {"logger": MyLogger(), "ignoreerrors": True}
        with YoutubeDL(ydl_opts) as ydl:
            info_dict = ydl.extract_info(search_term, download=False)
        return info_dict["entries"]
    except Exception as e:
        logger.error(
            "Error with getting the youtube url for %s : %s.", search_term, e
        )
        return None



from youtube_extract import ydl_utils
from youtube_extract import __main__ as ydl
import pytest
import sys


@pytest.fixture(scope="session")
def url():
    url = "https://www.youtube.com/channel/UCbYMTn6xKV0IKshL4pRCV3g/videos"
    return url


@pytest.fixture(scope="session")
def raw_entries(url):
    entries = ydl_utils.ydl_get_entries(url)
    return entries


@pytest.fixture(scope="session")
def entries(url):
    entries = ydl.extract_entries_for_url(url)
    return entries


@pytest.fixture(scope="session")
def args_simple():
    url = "https://www.youtube.com/channel/UCz4wfOcIw_OezAZTQ0SjiYA/videos"
    sys.argv = ["youtube_extract", url]
    args = ydl.parse_args()
    return args


@pytest.fixture(scope="session")
def args_complex():
    url = "https://www.youtube.com/channel/UCz4wfOcIw_OezAZTQ0SjiYA/videos"
    sys.argv = ["youtube_extract", url, "-e", "xlsx"]
    args = ydl.parse_args()
    return args


@pytest.fixture(scope="session")
def args_complex_2():
    url = "https://www.youtube.com/channel/UCz4wfOcIw_OezAZTQ0SjiYA/videos"
    sys.argv = ["youtube_extract", url, "-e", "csv"]
    args = ydl.parse_args()
    return args


@pytest.fixture(scope="session")
def args_complex_3():
    url = "https://www.youtube.com/channel/UCz4wfOcIw_OezAZTQ0SjiYA/videos"
    sys.argv = ["youtube_extract", "--export_format", "xlsx", url]
    args = ydl.parse_args()
    return args


@pytest.fixture(scope="session")
def args_complex_4():
    url = "https://www.youtube.com/channel/UCz4wfOcIw_OezAZTQ0SjiYA/videos"
    sys.argv = ["youtube_extract", "--export_format", "csv", url]
    args = ydl.parse_args()
    return args


@pytest.fixture(scope="session")
def args_incorrect():
    url = "https://www.youtube.com/channel/UCz4wfOcIw_OezAZTQ0SjiYA/videos"
    sys.argv = ["youtube_extract", "--export_format", "mp4", url]
    args = ydl.parse_args()
    return args


@pytest.fixture(scope="session")
def args_incorrect_2():
    sys.argv = ["youtube_extract", "--export_format", "xlsx"]
    args = ydl.parse_args()
    return args



from youtube_extract import __main__ as ydl
import pytest


def test_is_youtube_channel():
    urls_correct = [
        "https://www.youtube.com/channel/UCbYMTn6xKV0IKshL4pRCV3g/videos",
        "https://www.youtube.com/channel/UCz4wfOcIw_OezAZTQ0SjiYA/videos",
        "https://www.youtube.com/channel/UCz4wfOcIw_OezAZTQ0SjiYA",
    ]

    urls_incorrect = [
        "https://www.youtube.com",
        "https://www.youtubee.com",
        "https://www.youtubeee.com",
        "https://www.dbeley.ovh",
    ]

    for url in urls_correct:
        if not ydl.is_youtube_channel(url):
            raise AssertionError()

    for url in urls_incorrect:
        if ydl.is_youtube_channel(url):
            raise AssertionError()


def test_get_filename(entries):
    if not ydl.get_filename(entries) == "youtube_extract_Alex_Jimenez":
        raise AssertionError()


def test_check_args(
    args_simple,
    args_complex,
    args_complex_2,
    args_complex_3,
    args_complex_4,
    args_incorrect,
    args_incorrect_2,
):
    ydl.check_args(args_simple)
    ydl.check_args(args_complex)
    ydl.check_args(args_complex_2)
    ydl.check_args(args_complex_3)
    ydl.check_args(args_complex_4)
    with pytest.raises(Exception):
        ydl.check_args(args_incorrect)
    with pytest.raises(Exception):
        ydl.check_args(args_incorrect_2)


def test_extract_entries_for_url(url, entries):
    extracted_entries = ydl.extract_entries_for_url(url)
    print(entries)
    print(extracted_entries)
    if not extracted_entries == entries:
        raise AssertionError()

    print(len(extracted_entries))
    if len(extracted_entries) != 2:
        raise AssertionError()

    if extracted_entries[-1]["author"] != "Alex Jimenez":
        raise AssertionError()
    if extracted_entries[-1]["title"] != "color red":
        raise AssertionError()
    if extracted_entries[-1]["duration"] != 17:
        raise AssertionError()
    if extracted_entries[-1]["filesize_bytes"] != 150217:
        raise AssertionError()



import setuptools
import youtube_extract

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="youtube_extract",
    version=youtube_extract.__version__,
    author="dbeley",
    author_email="dbeley@protonmail.com",
    description="Extract metadata for all videos from a youtube channel into a csv or xlsx file.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dbeley/youtube_extract",
    packages=setuptools.find_packages(),
    include_package_data=True,
    entry_points={"console_scripts": ["youtube_extract=youtube_extract.__main__:main"]},
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: POSIX :: Linux",
    ],
    install_requires=["youtube-dl", "pandas", "openpyxl"],
)



"""
Extract metadata for all videos from a youtube channel into a csv file.
"""
import logging
import time
import argparse
from youtube_extract import ydl_utils
import pandas as pd

logger = logging.getLogger()
temps_debut = time.time()

SUPPORTED_EXPORT_FORMAT = ["csv", "xlsx"]


def is_youtube_channel(channel_url):
    if "youtube" not in channel_url:
        return False
    if not any(x in channel_url for x in ["/c/", "channel", "user"]):
        return False
    return True


def get_filename(list_dict):
    return f"youtube_extract_{list_dict[0]['author'].replace(' ', '_')}"


def check_args(args):
    if args.export_format not in SUPPORTED_EXPORT_FORMAT:
        raise Exception(
            f"{args.export_format} format not supported as export format. Exiting."
        )
    if not args.channel_url:
        raise Exception(
            f"No url set. Use youtube_extract CHANNEL_URL as command to input an URL."
        )

    if not is_youtube_channel(args.channel_url):
        raise Exception(
            f"{args.channel_url} is not a valid youtube channel url. Exiting."
        )


def extract_entries_for_url(channel_url):
    list_dict = []
    logger.debug("Extracting videos infos for %s.", channel_url)
    entries = ydl_utils.ydl_get_entries(channel_url)
    # workaround if channel videos are seen as a playlist
    if "_type" in entries[0]:
        if entries[0]["_type"] == "playlist":
            entries = entries[0]["entries"]
    for entry in entries:
        if entry:
            best_format = entry["formats"][-2]["format"]
            filesize = entry["formats"][-2]["filesize"]
            list_dict.append(
                {
                    "author": entry.get("uploader", ""),
                    "channel_url": entry.get("uploader_url", ""),
                    "title": entry.get("title", ""),
                    "webpage_url": entry.get("webpage_url", ""),
                    "view_count": entry.get("view_count", ""),
                    "like_count": entry.get("like_count", ""),
                    "dislike_count": entry.get("dislike_count", ""),
                    "average_rating": entry.get("average_rating", ""),
                    "duration": entry.get("duration", ""),
                    "upload_date": entry.get("upload_date", ""),
                    "tags": entry.get("tags", ""),
                    "categories": entry.get("categories", ""),
                    "description": entry.get("description", ""),
                    "thumbnail": entry.get("thumbnail", ""),
                    "best_format": best_format,
                    "filesize_bytes": filesize,
                }
            )
    return list_dict


def main():
    args = parse_args()
    logger.debug("youtube_extract : %s.", args)

    try:
        check_args(args)
    except Exception as e:
        logger.error(e)
        exit(1)

    entries = extract_entries_for_url(args.channel_url)
    export_filename = get_filename(entries)

    logger.debug("Exporting to %s.", export_filename)
    df = pd.DataFrame(entries)

    if args.export_format == "csv":
        df.to_csv(export_filename + ".csv", index=False, sep="\t")
    elif args.export_format in ["xls", "xlsx"]:
        df.to_excel(export_filename + ".xlsx", index=False)

    logger.info("Runtime : %.2f seconds." % (time.time() - temps_debut))
    exit(0)


def parse_args():
    format = "%(levelname)s :: %(message)s"
    parser = argparse.ArgumentParser(
        description="Extract metadata for all videos from a youtube channel into a csv or xlsx file."
    )
    parser.add_argument(
        "--debug",
        help="Display debugging information.",
        action="store_const",
        dest="loglevel",
        const=logging.DEBUG,
        default=logging.INFO,
    )
    parser.add_argument(
        "-e",
        "--export_format",
        type=str,
        help="Export format (csv or xlsx). Default : csv.",
        default="csv",
    ),
    parser.add_argument("channel_url", nargs="?", type=str, help="Youtube channel url.")
    args = parser.parse_args()

    logging.basicConfig(level=args.loglevel, format=format)
    return args


if __name__ == "__main__":
    main()
